﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Mango.Controls.Menu
{
    public partial class MenuBilan : UserControl
    {
    
        private ConnectionManager _connection = ConnectionManager.GetInstance();
    
        public MenuBilan()
        {
            InitializeComponent();
        }

        private void MenuBilan_Load(object sender, EventArgs e)
        {
            DataSet dataSet = _connection.DataSet;
            
            cboEvenement.DataSource = dataSet.Tables["Evenements"].Copy();
            cboEvenement.DisplayMember = "titreEvent";
            cboEvenement.ValueMember = "codeEvent";

            UpdateEvent(1);
        }
        
        /// <summary>
        /// Met à jour la page avec l'événement sélectionné
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        private void UpdateEvent(int eventId)
        {
            DataTable participants = _connection.GetParticipants(eventId);

            cboParticipant.DataSource = participants;
            cboParticipant.DisplayMember = "nomComp";
            cboParticipant.ValueMember = "codeParticipant";
            
            UpdateUser(eventId, (int) participants.Rows[0]["codeParticipant"]);
        }

        /// <summary>
        /// Met à jour la page avec l'utilisateur et l'événement sélectionné
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        /// <param name="userId">L'id de l'utilisateur</param>
        private void UpdateUser(int eventId, int userId)
        {
            dgvDepenses.Rows.Clear();
            dgvDettes.Rows.Clear();
            
            DataRow user = _connection.Filter("Participants", "codeParticipant = " + userId).Rows[0];
            
            //===========================
            // AFFICHER LES DEPENSES
            //===========================

            decimal total = 0;
            try
            {
                DataTable depenses = _connection.Filter("Depenses", "codeEvent = " + eventId + " AND codePart = " + userId);
                foreach (DataRow row in depenses.Rows)
                {
                    string date = DateTime.Parse(row["dateDepense"].ToString()).ToString("dd/MM/yyyy");
                    dgvDepenses.Rows.Add(row["description"], row["montant"] + " €", date, row["commentaire"]);
                }
                total = (decimal) depenses.Compute("Sum(montant)", "");
            }catch(ArgumentException) { }

            lblDepenses.Text = "Dépenses de " + user["prenomPart"];
            lblTotalDep.Text = "Total dépensé : " + total + "€";

            
            //===========================
            // AFFICHER LES DETTES
            //===========================

            decimal totalDu = 0;

            try
            {
                DataTable dettes = _connection.GetDepensesBeneficiaire(eventId, userId);
                foreach (DataRow row in dettes.Rows)
                {
                    long partsTotal = GetNbParts((int) row["numDepense"]);
                    string date = DateTime.Parse(row["dateDepense"].ToString()).ToString("dd/MM/yyyy");
                    string nom = _connection.Filter("Participants", "codeParticipant = " + row["codePart"])
                        .Rows[0]["nomComp"].ToString();
                    decimal montant = Math.Round((decimal) row["montant"] * (int) user["nbParts"] / partsTotal, 2);
                    totalDu += montant;
                    dgvDettes.Rows.Add(row["description"], montant + " €", date, nom, row["commentaire"]);
                }
            }catch(ArgumentException) { }

            lblDettes.Text = "Dettes de " + user["prenomPart"];
            lblTotalDu.Text = "Total dû : " + totalDu + "€";
        }

        private void cboEvenement_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(cboEvenement.SelectedValue is int id)
                UpdateEvent(id);
        }

        private void cboParticipant_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(cboParticipant.SelectedValue is int id)
                UpdateUser((int) cboEvenement.SelectedValue, id);
        }

        /// <summary>
        /// Retourne le nombre de parts total d'une dépense
        /// </summary>
        /// <param name="depenseId">L'id de la dépense</param>
        /// <returns>Le nombre de parts total</returns>
        private long GetNbParts(int depenseId)
        {
            DataTable beneficiaires = _connection.Filter("Beneficiaires", "numDepense = " + depenseId);
            bool first = true;
            string codes = "";
            foreach (DataRow row in beneficiaires.Rows)
            {
                if (!first) codes += ", ";
                else first = false;
                codes += row["codePart"];
            }
            return (long) _connection.DataSet.Tables["Participants"]
                .Compute("Sum(nbParts)", "codeParticipant IN (" + codes + ")");;
        }
            
        private void btnSolder_Click(object sender, EventArgs e)
        {
            FrmMain.GetInstance().ShowMenu(new MenuRecapitulatif((int)cboEvenement.SelectedValue));
        }
    }
}
