﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Mango.Controls.Menu
{
    public partial class MenuDepenses : UserControl
    {
        private ConnectionManager _connection = ConnectionManager.GetInstance();
        private DataSet _data;
        
        public MenuDepenses()
        {
            InitializeComponent();
            _data = _connection.DataSet;
        }

        private void MenuDepenses_Load(object sender, EventArgs e)
        {
            cboEvenement.DataSource = _data.Tables["Evenements"].Copy();
            cboEvenement.DisplayMember = "titreEvent";
            cboEvenement.ValueMember = "codeEvent";
            
            UpdateEvent(1);
        }

        private void UpdateEvent(int eventId)
        {
            try
            {
                cboParticipant.DataSource = _connection.GetParticipants(eventId);
                cboParticipant.DisplayMember = "nomComp";
                cboParticipant.ValueMember = "codeParticipant";
                cboParticipant.SelectedIndex = -1;

                DataTable depenses = _connection.Filter("Depenses", "codeEvent = " + eventId);
                lboDepenses.DataSource = depenses;
                lboDepenses.DisplayMember = "description";
                lboDepenses.ValueMember = "numDepense";
                lboDepenses.SelectedIndex = 0;
                UpdateSelectedDepense((int) depenses.Rows[0]["numDepense"]);
            }
            catch (ArgumentException)
            {
                lboDepenses.DataSource = null;
                lboDepenses.Items.Add("Aucune dépense");

                lblProprietes.Text = "Aucune dépense";
                lblCommentaire.Text = "";
                
                lboBeneficiaires.DataSource = null;
                lboBeneficiaires.Items.Clear();
            }
        }

        private void UpdateUser(int eventId, int userId)
        {
            try
            {
                lboDepenses.DataSource = _connection.Filter("Depenses", 
                    "codeEvent = " + eventId + " AND codePart = " + userId);
                lboDepenses.DisplayMember = "description";
                lboDepenses.ValueMember = "numDepense";
            }
            catch (ArgumentException)
            {
                lboDepenses.DataSource = null;
                lboDepenses.Items.Add("Aucune dépense");
            }
        }
        
        private void UpdateSelectedDepense(int depenseId)
        {
            DataRow depense = _connection.Filter("Depenses", "numDepense = " + depenseId).Rows[0];
            DataRow user = _connection.Filter("Participants", "codeParticipant = " + depense["codePart"]).Rows[0];

            lblProprietes.Text = depense["montant"] + "€ dépensés par " + user["nomComp"] + " le " 
                                 + DateTime.Parse(depense["dateDepense"].ToString()).ToString("dd/MM/yyyy");
            lblTitre.Text = depense["description"].ToString();
            lblCommentaire.Text = depense["commentaire"].ToString();

            try
            {
                lboBeneficiaires.DataSource = _connection.GetBeneficiaires(depenseId);
                lboBeneficiaires.DisplayMember = "nomComp";
                lboBeneficiaires.ValueMember = "codeParticipant";
            }catch(ArgumentException) { }

            try
            {
                cboAddBeneficiaire.DataSource =
                    _connection.GetNonBeneficiaires(depenseId, (int) cboEvenement.SelectedValue);
                cboAddBeneficiaire.DisplayMember = "nomComp";
                cboAddBeneficiaire.ValueMember = "codeParticipant";
            }
            catch (ArgumentException)
            {
                cboAddBeneficiaire.DataSource = null;
                cboAddBeneficiaire.SelectedIndex = -1;
            }
        }
        
        private void cboEvenement_SelectionChangeCommitted(object sender, EventArgs e)
        {
            UpdateEvent((int) cboEvenement.SelectedValue);
        }

        private void cboParticipant_SelectionChangeCommitted(object sender, EventArgs e)
        {
            UpdateUser((int) cboEvenement.SelectedValue, (int) cboParticipant.SelectedValue);
        }
        
        private void lboDepenses_Click(object sender, EventArgs e)
        {
            if (lboDepenses.DataSource != null)
                UpdateSelectedDepense((int) lboDepenses.SelectedValue);
        }

        private void btnNouvelleDep_Click(object sender, EventArgs e)
        {
            FrmMain.GetInstance().ShowMenu(new MenuNouvelleDepense((int) cboEvenement.SelectedValue));
        }

        private void btnAjouter_Click(object sender, EventArgs e)
        {
            try
            {
                Dictionary<string, object> values = new Dictionary<string, object>();
                values.Add("numDepense", lboDepenses.SelectedValue);
                values.Add("codePart", cboAddBeneficiaire.SelectedValue);
                OleDbCommand command = _connection.InsertInto("Beneficiaires", values);
                command.ExecuteNonQuery();
                _connection.FetchOffline();
                _data = _connection.DataSet;
                UpdateSelectedDepense((int) lboBeneficiaires.SelectedValue);
            }
            catch (OleDbException)
            {
                MessageBox.Show("Impossible d'ajouter le bénéficiaire !");
            }
        }
    }
}
