﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Mango.Controls.Menu
{
    public partial class MenuEvenement : UserControl
    {

        private ConnectionManager _connection = ConnectionManager.GetInstance();
        private DataTable _table; 
    
        public MenuEvenement()
        {
            InitializeComponent();
            _table = _connection.DataSet.Tables["Evenements"];

            lstEvent.DataSource = _table;
            lstEvent.DisplayMember = "titreEvent";
            lstEvent.ValueMember = "codeEvent";
            
            ShowEvent(1);
        }

        /// <summary>
        /// Met à jour la page avec l'événement sélectionné
        /// </summary>
        /// <param name="id">L'id de l'événement</param>
        public void ShowEvent(int id)
        {
            DataRow row = _connection.Filter("Evenements", "codeEvent = " + id).Rows[0];
            lblTitreEvent.Text = row["titreEvent"].ToString();
            lblSousTitre.Text = row["description"].ToString();
            DataRow user = _connection.Filter("Participants", "codeParticipant = " + row["codeCreateur"]).Rows[0];
            lblCreateur.Text = "Créé par " + user["prenomPart"] + " " + user["nomPart"];
            lblDebut.Text = "Commence le " + DateTime.Parse(row["dateDebut"].ToString()).ToString("dd/MM/yyyy");
            lblFin.Text = "Termine le " + DateTime.Parse(row["dateFin"].ToString()).ToString("dd/MM/yyyy");
            if ((bool)row["soldeON"])
            {
                lblStatut.Text = "Soldé !";
                lblStatut.ForeColor = Color.FromArgb(10, 195, 28);
            }
            else
            {
                lblStatut.Text = "Pas encore soldé...";
                lblStatut.ForeColor = Color.FromArgb(229, 5, 5);
            }
            lblNumEvent.Text = "Événement n°" + id;
            
            btnNext.Enabled = id < _table.Rows.Count;
            btnPrevious.Enabled = id > 1;

            lstEvent.SelectedIndex = id - 1;
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            FrmMain.GetInstance().ShowMenu(new MenuNouvelleEvent());
        }

        private void btnPrevious_Click(object sender, EventArgs e)
        {
            ShowEvent((int) lstEvent.SelectedValue - 1);
        }

        private void btnNext_Click(object sender, EventArgs e)
        {
            ShowEvent((int) lstEvent.SelectedValue + 1);
        }

        private void btnFirst_Click(object sender, EventArgs e)
        {
            ShowEvent(1);
        }

        private void btnLast_Click(object sender, EventArgs e)
        {
            ShowEvent(_table.Rows.Count);
        }

        private void lstEvent_Click(object sender, EventArgs e)
        {
            ShowEvent((int) lstEvent.SelectedValue);
        }
    }
}
