﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Mango.Controls.Menu
{
    public partial class MenuNouvelleDepense : UserControl
    {

        private ConnectionManager _connection = ConnectionManager.GetInstance();
        
        private readonly int _eventIndex;
        
        public MenuNouvelleDepense(int eventId)
        {
            DataTable events = _connection.Filter("Evenements", "NOT soldeON");
            for (int i = 0; i < events.Rows.Count ; i++)
            {
                if ((int) events.Rows[i]["codeEvent"] == eventId)
                    _eventIndex = i;
            }
            
            InitializeComponent();
        }

        private void MenuNouvelleDepense_Load(object sender, EventArgs e)
        {
            lblBienEnregistre.Visible = false;
            lblModifs.Visible = false;

            DataTable table = _connection.Filter("Evenements", "NOT soldeON");
            cboEvenement.DataSource = table;
            cboEvenement.DisplayMember = "titreEvent";
            cboEvenement.ValueMember = "codeEvent";
            cboEvenement.SelectedIndex = _eventIndex;
            
            UpdateEvent((int) table.Rows[0]["codeEvent"]);
        }

        /// <summary>
        /// Met à jour la page avec l'événement sélectionné
        /// </summary>
        /// <param name="id">L'id de l'événement</param>
        public void UpdateEvent(int id)
        {
            OleDbCommand command = _connection.CreateCommand(
                "SELECT prenomPart + ' ' + nomPart as nom, codePart as part FROM Participants p"+
                    " LEFT JOIN Invites i ON p.codeParticipant = i.codePart WHERE i.codeEvent = " + id);
            DataTable table = new DataTable();
            new OleDbDataAdapter(command).Fill(table);

            cboPayeur.DataSource = table.Copy();
            cboPayeur.DisplayMember = "nom";
            cboPayeur.ValueMember = "part";

            clbBeneficiaires.DataSource = table.Copy();
            clbBeneficiaires.DisplayMember = "nom";
        }

        private void cboEvenement_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(cboEvenement.SelectedValue is int)
                UpdateEvent((int) cboEvenement.SelectedValue);
        }

        private void btnRetour_Click(object sender, EventArgs e)
        {
            FrmMain.GetInstance().ShowMenu(new MenuDepenses());
        }

        private void btnEnregistrer_Click(object sender, EventArgs e)
        {
            if (CheckEntries())
            {
                bool ok = true;
                double montant = int.Parse(txtMontant.Text);

                Dictionary<string, object> values = new Dictionary<string, object>();

                OleDbCommand command = _connection.CreateCommand("BEGIN TRANSACTION");
                command.ExecuteNonQuery();

                int id = (int) _connection.CreateCommand("SELECT max(numDepense) FROM Depenses").ExecuteScalar() + 1;
                values.Add("numDepense", id);
                values.Add("description", txtDescription.Text);
                values.Add("montant", montant);
                values.Add("dateDepense", dtpDate.Value);
                if (txtCommentaire.Text.Length != 0)
                    values.Add("commentaire", txtCommentaire.Text);
                values.Add("codeEvent", cboEvenement.SelectedValue);
                values.Add("codePart", cboPayeur.SelectedValue);

                command = _connection.InsertInto("Depenses", values);
                ok = command.ExecuteNonQuery() == 1;

                foreach (DataRowView items in clbBeneficiaires.CheckedItems)
                {
                    values = new Dictionary<string, object>();
                    values.Add("numDepense", id);
                    values.Add("codePart", items["part"]);
                    command = _connection.InsertInto("Beneficiaires", values);
                    ok = command.ExecuteNonQuery() == 1 && ok;
                }

                if (ok)
                {
                    _connection.CreateCommand("COMMIT TRANSACTION").ExecuteNonQuery();
                    lblBienEnregistre.Visible = true;
                    timer.Enabled = true;
                    Cursor = Cursors.WaitCursor;

                    _connection.FetchOffline();
                }
                else _connection.CreateCommand("ROLLBACK TRANSACTION").ExecuteNonQuery();
            }
        }

        /// <summary>
        /// Vérifie les entrées de l'utilisateur
        /// </summary>
        /// <returns>Si les données sont valides</returns>
        private bool CheckEntries()
        {
            errorProvider.Clear();
            
            bool ok = true;
            if (clbBeneficiaires.CheckedItems.Count == 0)
            {
                errorProvider.SetError(clbBeneficiaires, "Aucun bénéficiaire sélectionné");
                errorProvider.SetIconAlignment(clbBeneficiaires, ErrorIconAlignment.MiddleLeft);
                errorProvider.SetIconPadding(clbBeneficiaires, 8);
                ok = false;
            }

            if (txtDescription.Text.Length == 0)
            {
                errorProvider.SetError(txtDescription, "Aucune description");
                errorProvider.SetIconPadding(txtDescription, 8);
                ok = false;
            }
            
            double montant;
            if (!double.TryParse(txtMontant.Text, out montant))
            {
                errorProvider.SetError(txtMontant, "Montant invalide");
                errorProvider.SetIconPadding(txtMontant, 28);
                ok = false;
            }

            return ok;
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            timer.Enabled = false;
            FrmMain.GetInstance().ShowMenu(new MenuAccueil());
        }

        private void chkTous_CheckedChanged(object sender, EventArgs e)
        {
            for (var i = 0; i < clbBeneficiaires.Items.Count; i++)
            {
                clbBeneficiaires.SetItemChecked(i, chkTous.Checked);
            }
        }

        private void btnRetour_MouseEnter(object sender, EventArgs e)
        {
            lblModifs.Visible = true;
        }

        private void btnRetour_MouseLeave(object sender, EventArgs e)
        {
            lblModifs.Visible = false;
        }
    }
}
