﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Data.OleDb;
using System.Net;
using System.Net.Mail;

namespace Mango.Controls.Menu
{
    public partial class MenuNouvelleEvent : UserControl
    {
        private ConnectionManager _connection = ConnectionManager.GetInstance();
        
        public MenuNouvelleEvent()
        { 
            InitializeComponent();
        }

        private void MenuNouvelleEvent_Load(object sender, EventArgs e)
        {
            DataSet dataSet = _connection.DataSet;
            cboCreation.DataSource = dataSet.Tables["Participants"].Copy();
            cboCreation.DisplayMember = "nomComp";
            cboCreation.ValueMember = "codeParticipant";
            
            clbInvites.DataSource = dataSet.Tables["Participants"].Copy();
            clbInvites.DisplayMember = "nomComp";
            clbInvites.ValueMember = "codeParticipant";

            lblBienEnregistre.Hide();
            lblModifs.Hide();
        }

        private void BtnBack_Click(object sender, EventArgs e)
        {
            FrmMain.GetInstance().ShowMenu(new MenuEvenement());
        }

        private void BtnSave_Click(object sender, EventArgs e)
        {
            if (CheckEntries())
            {
                bool ok = true;
                _connection.CreateCommand("BEGIN TRANSACTION").ExecuteNonQuery();
                
                Dictionary<string, object> values = new Dictionary<string, object>();
                int id = (int) _connection.CreateCommand("SELECT max(codeEvent) + 1 FROM Evenements").ExecuteScalar();
                values.Add("codeEvent", id);
                values.Add("titreEvent", txtIntitule.Text);
                values.Add("dateDebut", dtpDebut.Value.ToString("d"));
                values.Add("dateFin", dtpFin.Value.ToString("d"));
                values.Add("description", txtDescription.Text);
                values.Add("soldeON", false);
                values.Add("codeCreateur", cboCreation.SelectedValue);
                OleDbCommand command = _connection.InsertInto("Evenements", values);
                ok = command.ExecuteNonQuery() == 1;
                
                foreach (DataRowView item in clbInvites.CheckedItems)
                {
                    values = new Dictionary<string, object>();
                    values.Add("codeEvent", id);
                    values.Add("codePart", item["codeParticipant"]);
                    values.Add("login", "login");       // Valeurs aléatoires
                    values.Add("mdp", "mdp");           // puisque pas utilisées
                    command = _connection.InsertInto("Invites", values);
                    ok = command.ExecuteNonQuery() == 1 && ok;
                }

                if (ok)
                {
                    _connection.CreateCommand("COMMIT TRANSACTION").ExecuteNonQuery();
                    _connection.FetchOffline();
                    foreach (DataRowView item in clbInvites.CheckedItems)
                    {
                        MailManager.SendMail((string)item["adresseMail"], "Mango: Rejoignez l'événement " + txtIntitule.Text);
                    }
                    Cursor = Cursors.WaitCursor;
                    timer.Enabled = true;
                }
                else
                {
                    _connection.CreateCommand("ROLLBACK TRANSACTION").ExecuteNonQuery();
                    MessageBox.Show("Une erreur est survenue");
                }
            }
        }

        /// <summary>
        /// Vérifie si les entrées de l'utilisateur sont valides
        /// </summary>
        /// <returns>Si les donnés sont valides</returns>
        public bool CheckEntries()
        {
            errorProvider.Clear();
            
            bool ok = true;
            if (txtIntitule.Text.Length == 0)
            {
                errorProvider.SetError(txtIntitule, "Aucun intitulé");
                errorProvider.SetIconPadding(txtIntitule, 8);
                ok = false;
            }

            if (txtDescription.Text.Length == 0)
            {
                errorProvider.SetError(txtDescription, "Aucune description");
                errorProvider.SetIconPadding(txtDescription, 8);
                ok = false;
            }

            if (clbInvites.CheckedItems.Count == 0)
            {
                errorProvider.SetError(clbInvites, "Aucun invité");
                errorProvider.SetIconAlignment(clbInvites, ErrorIconAlignment.MiddleLeft);
                errorProvider.SetIconPadding(clbInvites, 8);
                ok = false;
            }

            if (dtpDebut.Value >= dtpFin.Value)
            {
                errorProvider.SetError(dtpFin, "Date avant le début de l'évenement");
                errorProvider.SetIconAlignment(dtpFin, ErrorIconAlignment.MiddleLeft);
                errorProvider.SetIconPadding(dtpFin, 8);
                ok = false;
            }
            
            return ok;
        }

        private void chkTous_CheckedChanged(object sender, EventArgs e)
        {
            for (var i = 0; i < clbInvites.Items.Count; i++)
            {
                clbInvites.SetItemChecked(i, chkTous.Checked);
            }
        }

        private void btnBack_MouseEnter(object sender, EventArgs e)
        {
            lblModifs.Show();
        }

        private void btnBack_MouseLeave(object sender, EventArgs e)
        {
            lblModifs.Hide();
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            timer.Stop();
            if (timer.Interval == 1000)
            {
                Cursor = Cursors.Default;
                lblBienEnregistre.Show();
                timer.Interval = 2000;
                timer.Start();
            }
            else
            {
                FrmMain.GetInstance().ShowMenu(new MenuEvenement());
            }
        }
    }
}
