﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Data.OleDb;

namespace Mango.Controls.Menu
{
    public partial class MenuParticipants : UserControl
    {
        private ConnectionManager _connection = ConnectionManager.GetInstance();
        private DataTable _tableEvent;
        private DataTable _tableParticipants;
        private DataTable _participantsEvenement;
        private DataTable _invitables;

        public MenuParticipants()
        {
            InitializeComponent();
        }

        private void MenuParticipants_Load(object sender, EventArgs e)
        {
            _tableEvent = _connection.DataSet.Tables["Evenements"];
            _tableParticipants = _connection.DataSet.Tables["Participants"];
            _participantsEvenement = new DataTable();
            _participantsEvenement.Columns.Add("Participant", typeof(string));
            _participantsEvenement.Columns.Add("Numéro de téléphone", typeof(string));
            _participantsEvenement.Columns.Add("Nombre de parts", typeof(Int32));
            _participantsEvenement.Columns.Add("Solde");

            _invitables = new DataTable();
            _invitables.Columns.Add("codeParticipant", typeof(int));
            _invitables.Columns.Add("nom", typeof(string));

            lstEvenements.DataSource = _tableEvent;
            lstEvenements.DisplayMember = "titreEvent";
            lstEvenements.ValueMember = "codeEvent";

            AfficherParticipants(1);
        }

        /// <summary>
        /// Met à jour la page avec les participants de l'événement sélectionné
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        private void AfficherParticipants(int eventId)
        {
            DataTable evenementCourant = _connection.Filter("Evenements", "codeEvent = " + eventId);
            lblEvenement.Text = evenementCourant.Rows[0]["titreEvent"].ToString();

            DataTable invites = _connection.Filter("Invites", "codeEvent = " + eventId);

            List<int> codesInvites = new List<int>();

            _participantsEvenement.Rows.Clear();
            _invitables.Rows.Clear();

            // FROM Invites
            for (int i = 0; i < invites.Rows.Count; i++)
            {
                // WHERE Invites.codeEvent = eventId
                if ((int)invites.Rows[i]["codeEvent"] == eventId)
                {
                    DataRow nouvelleLigne = _participantsEvenement.NewRow();
                    // JOIN Participants
                    for (int j = 0; j < _tableParticipants.Rows.Count; j++)
                    {
                        // ON Invites.codePart = Participants.codeParticipants
                        if ((int)invites.Rows[i]["codePart"] == (int)_tableParticipants.Rows[j]["codeParticipant"])
                        {
                            codesInvites.Add((int)_tableParticipants.Rows[j]["codeParticipant"]);
                            // SELECT Participants.prenomPart + ' ' + Participants.nomPart,
                            nouvelleLigne[0] = _tableParticipants.Rows[j]["prenomPart"].ToString() + " " + _tableParticipants.Rows[j]["nomPart"].ToString();
                            // Participants.mobile,
                            nouvelleLigne[1] = _tableParticipants.Rows[j]["mobile"].ToString();
                            // Participants.nbParts,
                            nouvelleLigne[2] = (int)_tableParticipants.Rows[j]["nbParts"];
                            // Participants.solde
                            nouvelleLigne[3] = _tableParticipants.Rows[j]["solde"].ToString();

                            _participantsEvenement.Rows.Add(nouvelleLigne);
                        }
                    }
                }
            }

            // on ne permet à l'utilisateur d'ajouter des invités que si l'événement n'est pas soldé
            if (!(bool)evenementCourant.Rows[0]["soldeON"])
            {
                cboInvites.Show();
                btnInviter.Show();
                lblAjouterParticipant.Text = "Ajouter un invité";
                //remplissage de la DataTable liée à la ComboBox avec les participants pas encore invités à l'événement
                foreach (DataRow participant in _tableParticipants.Rows)
                {
                    if (!codesInvites.Contains((int)participant["codeParticipant"]))
                    {
                        DataRow nouvelleLigne = _invitables.NewRow();

                        nouvelleLigne[0] = participant["codeParticipant"];
                        nouvelleLigne[1] = participant["prenomPart"].ToString() + " " + participant["nomPart"].ToString();

                        _invitables.Rows.Add(nouvelleLigne);
                    }
                }
            }
            else
            {
                cboInvites.Hide();
                btnInviter.Hide();
                lblAjouterParticipant.Text = "Cet événement étant déjà soldé, vous ne pouvez pas ajouter d'invité.";
            }

            dgvParticipants.DataSource = _participantsEvenement;
            dgvParticipants.Columns[0].Width = 175;

            cboInvites.DataSource = _invitables;
            cboInvites.DisplayMember = "nom";
            cboInvites.ValueMember = "codeParticipant";
        }
        private void lstEvenements_Click(object sender, EventArgs e)
        {
            AfficherParticipants((int)lstEvenements.SelectedValue);
        }

        private void btnInviter_Click(object sender, EventArgs e)
        {
            if (cboInvites.SelectedValue != null)
            {
                Dictionary<string, object> nouvelInvite = new Dictionary<string, object>();
                nouvelInvite.Add("codeEvent", (int)lstEvenements.SelectedValue);
                nouvelInvite.Add("codePart", (int)cboInvites.SelectedValue);
                nouvelInvite.Add("login", "login");
                nouvelInvite.Add("mdp", "mdp");

                int eventCourant = (int)lstEvenements.SelectedValue;

                _connection.InsertInto("Invites", nouvelInvite).ExecuteNonQuery();
                _connection.FetchOffline();

                btnInviter.Cursor = Cursors.WaitCursor;
                DataRow user = _connection.Filter("Participants", 
                    "codeParticipant = " + cboInvites.SelectedValue).Rows[0];
                DataRow evenement = _connection.Filter("Evenements",
                    "codeEvent = " + lstEvenements.SelectedValue).Rows[0];
                MailManager.SendMail(user["adresseMail"].ToString(), "Mango: Rejoignez l'événement " + evenement["titreEvent"]);
                btnInviter.Cursor = Cursors.Default;
                
                lstEvenements.SelectedValue = eventCourant;
                AfficherParticipants(eventCourant);
            }
        }
    }
}
